@@ -87,10 +87,9 @@ Edited Comments and GPL license
 #define __VCGLIB_MATH_BASE
 
 #include <float.h>
-#include <cmath>
-using namespace std;
-
+#include <math.h>
 #include <assert.h>
+#include <cmath>
 #include <limits>
 #include <algorithm>
 
@@ -198,8 +197,7 @@ template<class T> int IsNAN(T t) {    return _isnan(t) || (!_finite(t)); }
 #elif defined(__MINGW32__) // GCC
 template<class T> int IsNAN(T t) {    return std::isnan(t) || std::isinf(t); }
 #elif defined(__GNUC__) // GCC
-  template<class T> int IsNAN(T t) {    return std::isnan(t) || std::isinf(t); }
-
+template<class T> int IsNAN(T t) {    return std::isnan(t) || std::isinf(t); }
 #else // generic
 
 template<class T> int IsNAN(T t)
